@@ -1,10 +1,62 @@
 package Algorithm::Simplex;
-use strict;
-use warnings;
-use vars '$AUTOLOAD';    # Keep 'use strict' happy
-use Carp;
+use Moose;
+use namespace::autoclean;
+use Data::Dumper;
 
-our $VERSION = '0.26';
+our $VERSION = '0.36';
+
+has tableau => (
+    is       => 'rw',
+    isa      => 'ArrayRef[ArrayRef]',
+    required => 1,
+);
+
+has number_of_rows => (
+    is         => 'ro',
+    isa        => 'Int',
+    init_arg   => undef,
+    lazy_build => 1,
+);
+
+has number_of_columns => (
+    is         => 'ro',
+    isa        => 'Int',
+    init_arg   => undef,
+    lazy_build => 1,
+);
+
+has EPSILON => (
+    isa     => 'Num',
+    is      => 'ro',
+    default => 1e-13,
+);
+
+has MAXIMUM_PIVOTS => (
+    isa     => 'Int',
+    is      => 'rw',
+    default => 200,
+);
+
+has x_variables => (
+    isa        => 'ArrayRef[HashRef]',
+    is         => 'rw',
+    lazy_build => 1,
+);
+has 'y_variables' => (
+    isa        => 'ArrayRef[HashRef]',
+    is         => 'rw',
+    lazy_build => 1,
+);
+has u_variables => (
+    isa        => 'ArrayRef[HashRef]',
+    is         => 'rw',
+    lazy_build => 1,
+);
+has v_variables => (
+    isa        => 'ArrayRef[HashRef]',
+    is         => 'rw',
+    lazy_build => 1,
+);
 
 =head1 Name
 
@@ -29,90 +81,82 @@ study the I<solve_LP> subroutine.
 
 =head1 Methods
 
-=head2 new
+=head2 _build_number_of_rows 
 
-Create a new tableau which has a _tableau attribute that is a 
-ArrayRef[ArrayRef], i.e. two-dimensional array.
+set the number of rows
 
 =cut
 
-sub new {
-    my $class = $_[0];
-    bless { _tableau => $_[1], }, $class;
-}
-
-use Class::MethodMaker abstract => [
-    qw(
-      tableau_is_optimal
-      pivot
-      )
-];
-
-{
-    my %_attrs = (
-        _tableau           => 'read',
-        _number_of_rows    => 'read',
-        _number_of_columns => 'read',
-    );
-
-    sub _accessible {
-        my ( $self, $attr, $mode ) = @_;
-        $_attrs{$attr} =~ m{$mode};
-    }
-}
-
-sub Algorithm::Simplex::AUTOLOAD {
-    no strict "refs";
-    my ( $self, $newval ) = @_;
-
-    # Handle get_ methods
-    if ( $AUTOLOAD =~ m{.*::get(_\w+)} && $self->_accessible( $1, 'read' ) ) {
-        my $attr_name = $1;
-        *{$AUTOLOAD} = sub { return $_[0]->{$attr_name} };
-        return $self->{$attr_name};
-    }
-
-    # Otherwise a method has been called that doesn't exist
-    croak "No such method: $AUTOLOAD";
-}
-
-sub Algorithm::Simplex::DESTROY {
+sub _build_number_of_rows {
     my $self = shift;
+
+    return scalar @{ $self->tableau } - 1;
 }
 
-=head2 set_generic_variable_names_from_dimensions
+=head2 _build_number_of_columns 
 
-Create variable names: x1, x2 ... , y1, y2, ... , u1, u2 ... , v1, v2 ...
-
-Our variables are represented by:
-
-    x, y, u, and v 
-    
-as found in Nering and Tuckers' book. 
-
-x and y are for the primal LP while u and v belong to the dual LP.
+set the number of columns given the tableau matrix
 
 =cut
 
-sub set_generic_variable_names_from_dimensions {
+sub _build_number_of_columns {
     my $self = shift;
-    my ( @x, @y, @v, @u );
-    for my $i ( 0 .. $self->{_number_of_rows} - 1 ) {
-        my $tmp_num = $i + 1;
-        my $y       = 'y' . $tmp_num;
-        $self->{_y_variables}->[$i]->{'generic'} = $y;
-        my $v = 'v' . $tmp_num;
-        $self->{_v_variables}->[$i]->{'generic'} = $v;
-    }
-    for my $j ( 0 .. $self->{_number_of_columns} - 1 ) {
-        my $tmp_num = $j + 1;
-        my $x       = 'x' . $tmp_num;
-        $self->{_x_variables}->[$j]->{'generic'} = $x;
-        my $u = 'u' . $tmp_num;
-        $self->{_u_variables}->[$j]->{'generic'} = $u;
-    }
+
+    return scalar @{ $self->tableau->[0] } - 1;
 }
 
+=head2 _build_x_variables
+
+Set x variable names for the given tableau.
+
+=cut
+
+sub _build_x_variables {
+    my $self = shift;
+
+    my $x_vars;
+    for my $j ( 0 .. $self->number_of_columns - 1 ) {
+        my $x_index = $j + 1;
+        $x_vars->[$j]->{'generic'} = 'x' .$x_index;
+    }
+    return $x_vars;
+}
+
+sub _build_y_variables {
+    my $self = shift;
+
+    my $y_vars;
+    for my $i ( 0 .. $self->number_of_rows - 1 ) {
+        my $y_index = $i + 1;
+        $y_vars->[$i]->{'generic'} = 'y' . $y_index;
+    }
+    return $y_vars;
+}
+
+sub _build_u_variables {
+    my $self = shift;
+
+    my $u_vars;
+    for my $j ( 0 .. $self->number_of_columns - 1 ) {
+                my $u_index = $j + 1;
+        $u_vars->[$j]->{'generic'} = 'u' . $u_index;
+    }
+    return $u_vars;
+}
+
+sub _build_v_variables {
+    my $self = shift;
+
+    my $v_vars;
+    for my $i ( 0 .. $self->number_of_rows - 1 ) {
+        my $v_index = $i + 1;
+        $v_vars->[$i]->{'generic'} = 'v' . $v_index;
+    }
+    return $v_vars;
+}
+
+no Moose;
+
 =head2 get_bland_number_for
 
 Given a column number (which represents a u variable) build the bland number 
@@ -123,9 +167,10 @@ from the generic variable name.
 sub get_bland_number_for {
     my $self          = shift;
     my $variable_type = shift;
-    my $variables     = '_' . $variable_type . '_variables';
+    my $variables     = $variable_type . '_variables';
     my $index         = shift;
-    my $generic_name  = $self->{$variables}->[$index]->{'generic'};
+    my $generic_name  = $self->$variables->[$index]->{'generic'};
+
     $generic_name =~ m{(.)(\d+)};
     my $var = $1;
     my $num = $2;
@@ -224,7 +269,7 @@ sub min_index {
 =head2 exchange_pivot_variables
 
 Exchange the variables when the a pivot is done.  The method pivot does the
-algrebra while this method does the variable swapping.
+algrebra while this method does the variable swapping (and thus tracking).
 
 =cut
 
@@ -235,38 +280,15 @@ sub exchange_pivot_variables {
 
     # exchange variables based on $pivot_column_number and $pivot_row_number
     my $increasing_primal_variable =
-      $self->{_x_variables}->[$pivot_column_number];
-    my $zeroeing_primal_variable = $self->{_y_variables}->[$pivot_row_number];
-    $self->{_x_variables}->[$pivot_column_number] = $zeroeing_primal_variable;
-    $self->{_y_variables}->[$pivot_row_number] = $increasing_primal_variable;
-
-    my $increasing_dual_variable = $self->{_v_variables}->[$pivot_row_number];
-    my $zeroeing_dual_variable =
-      $self->{_u_variables}->[$pivot_column_number];
-    $self->{_v_variables}->[$pivot_row_number]    = $zeroeing_dual_variable;
-    $self->{_u_variables}->[$pivot_column_number] = $increasing_dual_variable;
-}
-
-=head2 set_number_of_rows_and_columns
-
-Given a tableau (matrix), determine its size.
-
-=cut
-
-
-sub set_number_of_rows_and_columns {
-    my $self = shift;
-
-    my @rows           = @{ $self->{_tableau} };
-    my $number_of_rows = @rows;
-    $number_of_rows -= 1;
-    $self->{_number_of_rows} = $number_of_rows;
-
-    my @columns           = @{ $self->{_tableau}->[0] };
-    my $number_of_columns = @columns;
-    $number_of_columns -= 1;
-    $self->{_number_of_columns} = $number_of_columns;
+      $self->x_variables->[$pivot_column_number];
+    my $zeroeing_primal_variable = $self->y_variables->[$pivot_row_number];
+    $self->x_variables->[$pivot_column_number] = $zeroeing_primal_variable;
+    $self->y_variables->[$pivot_row_number]    = $increasing_primal_variable;
 
+    my $increasing_dual_variable = $self->v_variables->[$pivot_row_number];
+    my $zeroeing_dual_variable   = $self->u_variables->[$pivot_column_number];
+    $self->v_variables->[$pivot_row_number]    = $zeroeing_dual_variable;
+    $self->u_variables->[$pivot_column_number] = $increasing_dual_variable;
 }
 
 =head2 get_row_and_column_numbers 
@@ -277,40 +299,32 @@ Get the dimensions of the tableau.
 
 sub get_row_and_column_numbers {
     my $self = shift;
-    return $self->{_number_of_rows}, $self->{_number_of_columns};
+    return $self->number_of_rows, $self->number_of_columns;
 }
 
 =head2 determine_bland_pivot_row_and_column_numbers
 
-Higher level functions that uses other to return the (bland) pivot point.
+Higher level function that uses others to return the (bland) pivot point.
 
 =cut
 
-
 sub determine_bland_pivot_row_and_column_numbers {
     my $self = shift;
 
-    #return "you";
     my @simplex_pivot_columns = $self->determine_simplex_pivot_columns;
-    my $tmp_out               = "pivot column: " . $simplex_pivot_columns[0];
-
-    #return $tmp_out;
     my $pivot_column_number =
       $self->determine_bland_pivot_column_number(@simplex_pivot_columns);
-
-    #return "pivot column number: $pivot_column_number";
     my ( $positive_ratios, $positive_ratio_row_numbers ) =
       $self->determine_positive_ratios($pivot_column_number);
-
-    #return "postive_ratio: $positive_ratios->[0]";
     my $pivot_row_number =
       $self->determine_bland_pivot_row_number( $positive_ratios,
         $positive_ratio_row_numbers );
 
-    #return "pivot row: $pivot_row_number";
     return ( $pivot_row_number, $pivot_column_number );
 }
 
+__PACKAGE__->meta->make_immutable;
+
 1;
 
 __END__
@@ -321,6 +335,9 @@ Mateu X. Hunter C<hunter@missoula.org>
 
 Strong design influence by George McRae at the University of Montana.
 
+#moose for solid assistance in the refactor: particularly _build_* approach 
+and PDL + Moose namespace management, 'inner'.
+
 =head1 License
 
 You may distribute this code under the same terms as Perl itself.
@@ -368,6 +385,20 @@ PDL
 
 =back
 
+=head1 Variables
+
+We have implicit variable names: x1, x2 ... , y1, y2, ... , u1, u2 ... , v1, v2 ...
+
+Our variables are represented by:
+
+    x, y, u, and v 
+    
+as found in Nering and Tuckers' book. 
+
+x and y are for the primal LP while u and v belong to the dual LP.
+
+These variable names are set during BUILD of the tableau object.
+
 =head1 Limitations
 
 The API is going to change.
