@@ -1,11 +1,9 @@
 package Algorithm::Simplex::Float;
-use strict;
-use base 'Algorithm::Simplex';
+use Moose;
+use namespace::autoclean;
+extends 'Algorithm::Simplex';
 
-use Readonly;
-Readonly my $EPSILON => 1e-14;
-
-my $one     =  1;
+my $one      =  1;
 my $neg_one = -1;
 
 =head1 Name
@@ -29,25 +27,25 @@ sub pivot {
 
     # Do tucker algebra on pivot row
     my $scale =
-      $one / ( $self->{_tableau}->[$pivot_row_number]->[$pivot_column_number] );
-    for my $j ( 0 .. $self->{_number_of_columns} ) {
-        $self->{_tableau}->[$pivot_row_number]->[$j] =
-          $self->{_tableau}->[$pivot_row_number]->[$j] * ($scale);
+      $one / ( $self->tableau->[$pivot_row_number]->[$pivot_column_number] );
+    for my $j ( 0 .. $self->number_of_columns ) {
+        $self->tableau->[$pivot_row_number]->[$j] =
+          $self->tableau->[$pivot_row_number]->[$j] * ($scale);
     }
-    $self->{_tableau}->[$pivot_row_number]->[$pivot_column_number] = $scale;
+    $self->tableau->[$pivot_row_number]->[$pivot_column_number] = $scale;
 
     # Do tucker algebra elsewhere
-    for my $i ( 0 .. $self->{_number_of_rows} ) {
+    for my $i ( 0 .. $self->number_of_rows ) {
         if ( $i != $pivot_row_number ) {
 
             my $neg_a_ic =
-              $self->{_tableau}->[$i]->[$pivot_column_number] * ($neg_one);
-            for my $j ( 0 .. $self->{_number_of_columns} ) {
-                $self->{_tableau}->[$i]->[$j] =
-                  $self->{_tableau}->[$i]->[$j] + (
-                    $neg_a_ic * ( $self->{_tableau}->[$pivot_row_number]->[$j] ) );
+              $self->tableau->[$i]->[$pivot_column_number] * ($neg_one);
+            for my $j ( 0 .. $self->number_of_columns ) {
+                $self->tableau->[$i]->[$j] =
+                  $self->tableau->[$i]->[$j] + (
+                    $neg_a_ic * ( $self->tableau->[$pivot_row_number]->[$j] ) );
             }
-            $self->{_tableau}->[$i]->[$pivot_column_number] = $neg_a_ic * ($scale);
+            $self->tableau->[$i]->[$pivot_column_number] = $neg_a_ic * ($scale);
         }
     }
 }
@@ -60,14 +58,14 @@ This is how we decide when to stop the algorithm.
 
 =cut
 
-sub tableau_is_optimal {
+sub is_optimal {
     my $self = shift;
 
     # check basement row for having non-positive entries which would => optimal 
     # when in phase 2.  Use EPSILON instead of zero because we're dealing with floats
     my $optimal_flag = 1;
-    for my $j ( 0 .. $self->{_number_of_columns}-1 ) {
-        if ( $self->{_tableau}->[ $self->{_number_of_rows} ]->[$j] > $EPSILON ) {
+    for my $j ( 0 .. $self->number_of_columns-1 ) {
+        if ( $self->tableau->[ $self->number_of_rows ]->[$j] > $self->EPSILON ) {
             $optimal_flag = 0;
             last;
         }
@@ -95,8 +93,8 @@ sub determine_simplex_pivot_columns {
     # when the cost is zero.  This would not lead us to optimality, but if we were
     # already in an optimal state if may (should) lead to another optimal state.
     # This would only apply then in the optimal case, i.e. all entries non-positive.
-    for my $col_num ( 0 .. $self->{_number_of_columns} - 1 ) {
-        if ( $self->{_tableau}->[ $self->{_number_of_rows} ]->[$col_num] > $EPSILON )
+    for my $col_num ( 0 .. $self->number_of_columns - 1 ) {
+        if ( $self->tableau->[ $self->number_of_rows ]->[$col_num] > $self->EPSILON )
         {
             push( @simplex_pivot_column_numbers, $col_num );
         }
@@ -122,14 +120,14 @@ sub determine_positive_ratios {
     
 
     #print "Column: $possible_pivot_column\n";
-    for my $row_num ( 0 .. $self->{_number_of_rows} - 1 ) {
-        if ( $self->{_tableau}->[$row_num]->[$pivot_column_number] > $EPSILON )
+    for my $row_num ( 0 .. $self->number_of_rows - 1 ) {
+        if ( $self->tableau->[$row_num]->[$pivot_column_number] > $self->EPSILON )
         {
             push(
                 @positive_ratios,
-                    $self->{_tableau}->[$row_num]->[ $self->{_number_of_columns} ]
+                    $self->tableau->[$row_num]->[ $self->number_of_columns ]
                     / 
-                    $self->{_tableau}->[$row_num]->[$pivot_column_number]
+                    $self->tableau->[$row_num]->[$pivot_column_number]
                   );
 
             # Track the rows that give ratios
@@ -140,5 +138,6 @@ sub determine_positive_ratios {
     return (\@positive_ratios, \@positive_ratio_row_numbers);
 }
 
+__PACKAGE__->meta->make_immutable;
 
 1;
