@@ -1,6 +1,4 @@
 #!/usr/bin/env perl
-use strict;
-use warnings;
 use Benchmark;
 use PDL;
 use Algorithm::Simplex;
